/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#ifndef __GAMEWIDGET_HEADER__
#define __GAMEWIDGET_HEADER__

#include <qwidget.h>
#include <qpixmap.h>
#include <qtimer.h>

class GameWidget : public QWidget
{
    Q_OBJECT

public:
    GameWidget(QWidget *parent, const char *name = 0, WFlags f = 0);
    ~GameWidget();

private:
    static const int max_stars = 30;
    static const int max_bullets = 3;
    static const int max_mines = 20;
    static const int max_enemies = 10;
    static const int max_explosions = 32;
    static const int max_bonuses = 3;
    static const int max_gun = 100;
    static const int max_shield = 100;
    static const int max_fuel = 100;
    static const int max_bombs = 3;
    static const int max_lives = 6;
    static const int num_beams = 9;
    static const int num_counter = 100;

private:
    QTimer *m_timer;
    QPixmap *m_pixmap;
    QPixmap m_text[2];
    QPixmap m_logo[2];
    QPixmap m_bomb_pm[2];
    QPixmap m_bonus_pm[4][5];
    QPixmap m_bullet_pm[4]; 
    QPixmap m_mine_pm[4]; 
    QPixmap m_glider_pm[2];
    QPixmap m_enemy_pm[6];
    QPixmap m_big_expl_pm[23];
    QPixmap m_small_expl_pm[23];
    QPixmap m_mothership_pm;
    bool m_lefthand;
    bool m_mouse_down;
    int m_mouse_counter;
    int m_max_y;
    int m_max_x;
    enum States {
	Intro = 0,
	Playing,
	Dying,
	SafeMode,
	GameOver,
    } m_state;
    int m_counter;
    int m_beam_counter;
    int m_bomb_counter;
    int m_bomb_x;
    int m_bomb_y;
    int m_bomb_diff_x;
    int m_bomb_diff_y;
    int m_req_fire;
    struct Star {
	int x;
	int y;
	int size;
	int color;
    } m_star[max_stars];
    struct Bullet {
	int beam;
	int counter;
    } m_bullet[max_bullets];
    struct Mine {
	int beam;
	int counter;
    } m_mine[max_mines];
    struct Enemy {
	int x;
	int req_x;
	int req_end_x;
	int beam;
	int counter;
	int req_counter;
    } m_enemy[max_enemies];
    struct Coord {
	int x;
	int y;
    } m_coord[num_beams][num_counter];
    struct Explosion {
	int x;
	int y;
	int big;
	int counter;
    } m_explosion[max_explosions];
    struct Bonus {
	int beam;
	int counter;
	int what;
    } m_bonus[max_bonuses];
    int m_bonuses;
    int m_explosions;
    int m_bullets;
    int m_mines;
    int m_enemies;
    int m_gun;
    int m_shield;
    int m_fuel;
    int m_bombs;
    int m_lives;
    int m_score;
    int m_enemies_killed;
    int m_motherships_killed;
    int m_bullets_fired;
    int m_bombs_fired;
    struct Beam {
	int far_x;
	int near_x;
    } m_beam[num_beams];
    int m_far_y;
    int m_near_y;
    int m_player_x;
    int m_player_req_x;
    int m_player_req_beam;
    int m_mothership_x;
    int m_mothership_req_x;
	
public slots:
    void timerTick(void);

protected:
    void paintEvent(QPaintEvent *event);
    void showEvent(QShowEvent *event);
    void hideEvent(QHideEvent *event);
    void resizeEvent(QResizeEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);

private:
    void drawPixmapText(QPainter &painter, int x, int y, int wrap, const char *str);
    void drawPixmapTextCentered(QPainter &painter, int x, int y, int wrap, const char *str);
    void drawPixmapTextRightJustified(QPainter &painter, int x, int y, int wrap, const char *str);
    void drawBar(QPainter &painter, int x, int y, int width, int value, int red, int yellow, int max);
    void readConfig(void);
    void writeConfig(void);
    void moveLeft(void);
    void moveRight(void);
    void fireBullet(void);
    void deleteBullet(int i);
    void dropMine(int beam, int counter);
    void deleteMine(int i);
    void makeExplosion(int x, int y, int big);
    void deleteExplosion(int i);
    void makeBonus(int beam, int counter, int always);
    void deleteBonus(int i);
    void makeEnemy(void);
    void deleteEnemy(int i);
    void dropBomb(void);
    void flipScreen(void);
    void switchState(enum States state);
    void start(void);
    void stop(void);
    void drawIntro(void);
    void drawPlaying(void);
    void drawGameOver(void);
    void makeStars(void);
    void drawStars(QPainter &painter);
    void calculateBeams(void);
    void drawBeams(QPainter &painter);
    void drawStats(QPainter &painter);
    void drawShip(QPainter &painter);
    void drawBullets(QPainter &painter);
    void drawBomb(QPainter &painter);
    void drawMines(QPainter &painter);
    void drawEnemies(QPainter &painter);
    void drawMothership(QPainter &painter);
    void drawBonuses(QPainter &painter);
    void drawExplosions(QPainter &painter);
    int getXCoord(int beam, int counter);
    int getYCoord(int beam, int counter);
    void updateActors(void);
    void addScore(int score);
};

#endif // __GAMEWIDGET_HEADER__

